@@ -7,6 +7,7 @@ from .config import Config
 from .errors import Error, MethodError, NotSdistError, NotVCSError, NotVersioningitError
 from .logging import log, warn_bad_version
 from .methods import VersioningitMethod
+from .onbuild import OnbuildFileProvider, SetuptoolsFileProvider
 from .util import is_sdist, parse_version_from_metadata
 
 if TYPE_CHECKING:
@@ -25,8 +26,9 @@ class VCSDescription:
     #: The relationship of the repository's current state to the tag.  If the
     #: repository state is exactly the tagged state, this field should equal
     #: ``"exact"``; otherwise, it will be a string that will be used as a key
-    #: in the ``[tool.versioningit.format]`` subtable.  Recommended values are
-    #: ``"distance"``, ``"dirty"``, and ``"distance-dirty"``.
+    #: in the ``format`` subtable of the versioningit configuration.
+    #: Recommended values are ``"distance"``, ``"dirty"``, and
+    #: ``"distance-dirty"``.
     state: str
 
     #: The name of the repository's current branch, or `None` if it cannot be
@@ -146,20 +148,21 @@ class Versioningit:
         ``project_dir`` (default: the current directory).
 
         If ``config`` is `None`, then ``project_dir`` must contain a
-        :file:`pyproject.toml` file containing a ``[tool.versioningit]`` table;
-        if it does not, a `NotVersioningitError` is raised.  If ``config`` is
-        not `None`, then any :file:`pyproject.toml` file in ``project_dir``
-        will be ignored, and the configuration will be taken from ``config``
-        instead.  See ":ref:`config_dict`".
+        :file:`pyproject.toml` file containing either a ``[tool.versioningit]``
+        table or a ``[tool.hatch.version]`` table with the ``source`` key set
+        to ``"versioningit"``; if it does not, a `NotVersioningitError` is
+        raised.  If ``config`` is not `None`, then any :file:`pyproject.toml`
+        file in ``project_dir`` will be ignored, and the configuration will be
+        taken from ``config`` instead.  See ":ref:`config_dict`".
 
         :raises NotVersioningitError:
             - if ``config`` is `None` and ``project_dir`` does not contain a
               :file:`pyproject.toml` file
             - if ``config`` is `None` and the :file:`pyproject.toml` file does
-              not contain a ``[tool.versioningit]`` table
+              not contain a versioningit configuration table
         :raises ConfigError:
-            if the ``tool.versioningit`` key, ``config``, or any subfields are
-            not of the correct type
+            if the configuration object/table or any of its subfields are not
+            of the correct type
         """
         if config is None:
             try:
@@ -187,17 +190,17 @@ class Versioningit:
             format=config.format.load(project_dir),
             template_fields=config.template_fields.load(project_dir),
             write=config.write.load(project_dir) if config.write is not None else None,
-            onbuild=config.onbuild.load(project_dir)
-            if config.onbuild is not None
-            else None,
+            onbuild=(
+                config.onbuild.load(project_dir) if config.onbuild is not None else None
+            ),
         )
 
     def get_version(self, write: bool = False, fallback: bool = True) -> str:
         """
         Determine the version for the project.
 
-        If ``write`` is true, then the file specified in the
-        ``[tool.versioningit.write]`` subtable, if any, will be updated.
+        If ``write`` is true, then the file specified in the ``write`` subtable
+        of the versioningit configuration, if any, will be updated.
 
         If ``fallback`` is true, then if ``project_dir`` is not under version
         control (or if the VCS executable is not installed), ``versioningit``
@@ -230,8 +233,8 @@ class Versioningit:
         optionally, "write" — and return an object containing the final version
         and intermediate values.
 
-        If ``write`` is true, then the file specified in the
-        ``[tool.versioningit.write]`` subtable, if any, will be updated.
+        If ``write`` is true, then the file specified in the ``write`` subtable
+        of the versioningit configuration, if any, will be updated.
 
         If ``fallback`` is true, then if ``project_dir`` is not under version
         control (or if the VCS executable is not installed), ``versioningit``
@@ -281,7 +284,7 @@ class Versioningit:
                 )
             if self.default_version is not None:
                 log.error("%s: %s", type(e).__name__, str(e))
-                log.info("Falling back to tool.versioningit.default-version")
+                log.info("Falling back to default-version")
                 version = self.default_version
                 using_default_version = True
             else:
@@ -289,7 +292,7 @@ class Versioningit:
         except Exception:  # pragma: no cover
             if self.default_version is not None:
                 log.exception("An unexpected error occurred:")
-                log.info("Falling back to tool.versioningit.default-version")
+                log.info("Falling back to default-version")
                 version = self.default_version
                 using_default_version = True
             else:
@@ -424,7 +427,7 @@ class Versioningit:
 
     def do_onbuild(
         self,
-        build_dir: str | Path,
+        file_provider: OnbuildFileProvider,
         is_source: bool,
         template_fields: dict[str, Any],
     ) -> None:
@@ -436,10 +439,14 @@ class Versioningit:
         .. versionchanged:: 2.0.0
 
             ``version`` argument replaced with ``template_fields``
+
+        .. versionchanged:: 3.0.0
+
+            ``build_dir`` argument replaced with ``file_provider``
         """
         if self.onbuild is not None:
             self.onbuild(
-                build_dir=build_dir,
+                file_provider=file_provider,
                 is_source=is_source,
                 template_fields=template_fields,
             )
@@ -457,14 +464,15 @@ def get_version(
     Determine the version for the project at ``project_dir``.
 
     If ``config`` is `None`, then ``project_dir`` must contain a
-    :file:`pyproject.toml` file containing a ``[tool.versioningit]`` table; if
-    it does not, a `NotVersioningitError` is raised.  If ``config`` is not
-    `None`, then any :file:`pyproject.toml` file in ``project_dir`` will be
-    ignored, and the configuration will be taken from ``config`` instead; see
-    ":ref:`config_dict`".
+    :file:`pyproject.toml` file containing either a ``[tool.versioningit]``
+    table or a ``[tool.hatch.version]`` table with the ``source`` key set to
+    ``"versioningit"``; if it does not, a `NotVersioningitError` is raised.  If
+    ``config`` is not `None`, then any :file:`pyproject.toml` file in
+    ``project_dir`` will be ignored, and the configuration will be taken from
+    ``config`` instead.  See ":ref:`config_dict`".
 
-    If ``write`` is true, then the file specified in the
-    ``[tool.versioningit.write]`` subtable, if any, will be updated.
+    If ``write`` is true, then the file specified in the ``write`` subtable of
+    the versioningit configuration, if any, will be updated.
 
     If ``fallback`` is true, then if ``project_dir`` is not under version
     control (or if the VCS executable is not installed), ``versioningit`` will
@@ -480,8 +488,8 @@ def get_version(
     :raises NotVersioningitError:
         - if ``config`` is `None` and ``project_dir`` does not contain a
           :file:`pyproject.toml` file
-        - if the :file:`pyproject.toml` file does not contain a
-          ``[tool.versioningit]`` table
+        - if ``config`` is `None` and the :file:`pyproject.toml` file does not
+          contain a versioningit configuration table
     :raises ConfigError:
         if any of the values in ``config`` are not of the correct type
     :raises MethodError: if a method returns a value of the wrong type
@@ -500,19 +508,20 @@ def get_next_version(
     ``project_dir``.
 
     If ``config`` is `None`, then ``project_dir`` must contain a
-    :file:`pyproject.toml` file containing a ``[tool.versioningit]`` table; if
-    it does not, a `NotVersioningitError` is raised.  If ``config`` is not
-    `None`, then any :file:`pyproject.toml` file in ``project_dir`` will be
-    ignored, and the configuration will be taken from ``config`` instead; see
-    ":ref:`config_dict`".
+    :file:`pyproject.toml` file containing either a ``[tool.versioningit]``
+    table or a ``[tool.hatch.version]`` table with the ``source`` key set to
+    ``"versioningit"``; if it does not, a `NotVersioningitError` is raised.  If
+    ``config`` is not `None`, then any :file:`pyproject.toml` file in
+    ``project_dir`` will be ignored, and the configuration will be taken from
+    ``config`` instead.  See ":ref:`config_dict`".
 
     :raises NotVCSError:
         if ``project_dir`` is not under version control
     :raises NotVersioningitError:
         - if ``config`` is `None` and ``project_dir`` does not contain a
           :file:`pyproject.toml` file
-        - if the :file:`pyproject.toml` file does not contain a
-          ``[tool.versioningit]`` table
+        - if ``config`` is `None` and the :file:`pyproject.toml` file does not
+          contain a versioningit configuration table
     :raises ConfigError:
         if any of the values in ``config`` are not of the correct type
     :raises MethodError: if a method returns a value of the wrong type
@@ -553,12 +562,12 @@ def run_onbuild(
     """
     .. versionadded:: 1.1.0
 
-    Run the ``onbuild`` step for the given project.
+    Run the ``onbuild`` step for the given setuptools project.
 
-    This function is intended to be used by custom setuptools commands that are
-    used in place of ``versioningit``'s custom commands but still need to be
-    able to run the ``onbuild`` step.  The ``template_fields`` value can be
-    obtained by passing the command's ``distribution`` attribute to
+    This function is intended to be used by custom setuptools command classes
+    that are used in place of ``versioningit``'s command classes but still need
+    to be able to run the ``onbuild`` step.  The ``template_fields`` value can
+    be obtained by passing a command class's ``distribution`` attribute to
     `get_template_fields_from_distribution()`; if this returns `None`, then we
     are building from an sdist, and `run_onbuild()` should not be called.
 
@@ -590,7 +599,9 @@ def run_onbuild(
     """
     vgit = Versioningit.from_project_dir(project_dir, config)
     vgit.do_onbuild(
-        build_dir=build_dir, is_source=is_source, template_fields=template_fields
+        file_provider=SetuptoolsFileProvider(build_dir=Path(build_dir)),
+        is_source=is_source,
+        template_fields=template_fields,
     )
 
 
